==> archibald-7.x-2.0.tar.gz: archibald/includes/file_uploader/uploader.ftp.class.php <==
<?php

/**
 * @file
 * Provides the FTP upload handler
 */

/**
 * file uploader to educa file server
 * used by lom publish
 *
 * @author Heiko Henning h.henning@educa.ch
 */
class ArchibaldUploaderFtp extends ArchibaldUploaderCurlBased implements ArchibaldUploaderInterface {

  /**
   * Error constances
   */
  const ERROR_INVALID_PORT = 0;
  const ERROR_INVALID_CONNECTION_TIMEOUT = 1;
  const ERROR_CONNECTION_TIMEOUT = 2;
  const ERROR_INVALID_CREDENTIALS = 3;
  const ERROR_INVALID_UPLOAD_PATH = 4;
  const ERROR_PERMISSION_CREATE_DIR = 5;
  const ERROR_PERMISSION_REMOVE_DIR = 6;
  const ERROR_PERMISSION_UPLOAD_FILE = 7;
  const ERROR_PERMISSION_DELETE_FILE = 8;

  /**
   * get the name of the uploader
   *
   * @return array
   *  first value is the computer readable code
   *  second value is the human readable text
   */
  public function getUploaderName() {
    return array('ftp' => t('FTP Server'));
  }

  /**
   * get the description of the uploader
   *
   * @return string
   *   html
   */
  public function getUploaderDescription() {
    return t('Uploads files to an configured ftp server.');
  }

  /**
   * get settings configuration form
   *
   * @param array $form
   * @param array $form_state
   *
   * @return array
   *   drupal form
   */
  public function getSettingsForm($form, &$form_state, $content_partner_id) {

    $saved_values = array_merge(
      array(
        'server' => 'localhost',
        'port' => '21',
        'ssl' => '0',
        'timeout' => '60',
        'username' => '',
        'password' => '',
        'passive' => '0',
        'upload_path' => '/',
        'public_path' => '/',
      ),
      (array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id, array())
    );

    $settings_form['server'] = array(
      '#type' => 'textfield',
      '#title' => t('Servername/IP-Address'),
      '#required' => TRUE,
      '#default_value' => $saved_values['server'],
    );


    $settings_form['port'] = array(
      '#type' => 'textfield',
      '#title' => t('Port'),
      '#default_value' => $saved_values['port'],
    );

    $settings_form['ssl'] = array(
      '#type' => 'checkbox',
      '#title' => t('SSL'),
      '#return_value' => 1,
      '#default_value' => $saved_values['ssl'],
    );

    $settings_form['passive'] = array(
      '#type' => 'checkbox',
      '#title' => t('Passive-mode'),
      '#return_value' => 1,
      '#default_value' => $saved_values['passive'],
    );

    $settings_form['timeout'] = array(
      '#type' => 'textfield',
      '#title' => t('Connection timeout'),
      '#default_value' => $saved_values['timeout'],
    );


    $settings_form['username'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#default_value' => $saved_values['username'],
    );

    $settings_form['password'] = array(
      '#type' => 'textfield',
      '#title' => t('Password'),
      '#default_value' => $saved_values['password'],
    );

    $settings_form['upload_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Upload path on FTP server'),
      '#required' => TRUE,
      '#default_value' => $saved_values['upload_path'],
    );


    $settings_form['public_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Public URL and path on FTP server (example: http://my-url.ftp.info/files/'),
      '#description' => 'Please enter the web based public URL and path where the files will be available after uploading',
      '#required' => TRUE,
      '#default_value' => $saved_values['public_path'],
    );

    return $settings_form;
  }

  /**
   * validate configuration form
   *
   * @param array $form
   * @param array $form_state
   */
  public function validateSettingsForm($form, &$form_state) {
    if (!isset($form_state['port'])) {
      return;
    }

    if (empty($form_state['username']) && empty($form_state['password'])) {
      $form_state['username'] = "anonymous";
      $form_state['password'] = "anonymous@anonymous.de";
    }

    $validate_result = $this->validateLogin(
      $form_state['server'], $form_state['port'], $form_state['timeout'], $form_state['username'], $form_state['password'], $form_state['upload_path'], $form_state['passive'], $form_state['ssl']
    );

    if ($validate_result !== TRUE) {
      switch ($validate_result) {
        case self::ERROR_INVALID_PORT:
          return form_set_error('file_uploader_config][port', t("Invalid port, please use just numbers"));

        case self::ERROR_INVALID_CONNECTION_TIMEOUT:
          return form_set_error('file_uploader_config][timeout', t("Invalid connection timeout, please use just numbers"));

        case self::ERROR_CONNECTION_TIMEOUT:
          return form_set_error('file_uploader_config', t("Can not connect to server, please check your server settings"));

        case self::ERROR_INVALID_CREDENTIALS:

          return form_set_error('file_uploader_config', t("Invalid credentials, please check your username and password"));

        case self::ERROR_INVALID_UPLOAD_PATH:
          return form_set_error('file_uploader_config][upload_path', t("Could not find upload path, please check your settings"));

        case self::ERROR_PERMISSION_CREATE_DIR:
          return form_set_error('file_uploader_config][upload_path', t("The current user has no permission to create directories at the specific upload path, please check your settings"));

        case self::ERROR_PERMISSION_REMOVE_DIR:
          return form_set_error('file_uploader_config][upload_path', t("The current user has no permission to delete directories at the specific upload path, please check your settings"));

        case self::ERROR_PERMISSION_UPLOAD_FILE:
          return form_set_error('file_uploader_config][upload_path', t("The current user has no permission to upload files at the specific upload path, please check your settings"));

        case self::ERROR_PERMISSION_DELETE_FILE:
          return form_set_error('file_uploader_config][upload_path', t("The current user has no permission to delete files at the specific upload path, please check your settings"));

        default:
          return form_set_error('file_uploader_config', t("An unknown error occured"));
      }
    }

    if (!preg_match("/^https?:\/\/(www.)?/", $form_state['public_path'])) {
      return form_set_error('file_uploader_config][public_path', t("The public path must be a full url including http(s)://"));
    }
  }

  /**
   *
   * @param string $server the servername or the server address
   * @param int $port the port number
   * @param int $timeout the timeout for connecting
   * @param string $username the username
   * @param string $password the password
   * @param string $upload_path the path where to upload
   * @param boolean $passive TRUE to use passive mode
   * @param boolean $ssl TRUE to enable ssl
   *
   * @return mixed TRUE on success or the error number as an integer (one of uploader_ftp::ERROR_*)
   */
  private function validateLogin($server, $port, $timeout, $username, $password, $upload_path, $passive, $ssl) {
    if ((INT) $port . "" != $port . "") {
      return self::ERROR_INVALID_PORT;
    }

    if ((INT) $timeout . "" != $timeout . "") {
      return self::ERROR_INVALID_CONNECTION_TIMEOUT;
    }

    //server connection validation
    if (!empty($ssl)) {
      $ftp_stream = @ftp_ssl_connect($server, $port, $timeout);
    }
    else {
      $ftp_stream = @ftp_connect($server, $port, $timeout);
    }

    if (!$ftp_stream) {
      return self::ERROR_CONNECTION_TIMEOUT;
    }

    //set passive mode validation
    if (!empty($passive)) {
      ftp_pasv($ftp_stream, TRUE);
    }

    //credentials validation
    if (!@ftp_login($ftp_stream, $username, $password)) {
      return self::ERROR_INVALID_CREDENTIALS;
    }

    //upload path exist validation
    if (!@ftp_chdir($ftp_stream, $upload_path)) {
      return self::ERROR_INVALID_UPLOAD_PATH;
    }

    //Create directory validation
    $perm_check = @ftp_mkdir($ftp_stream, $upload_path . '/archibald_upload_check');
    if (empty($perm_check)) {
      return self::ERROR_PERMISSION_CREATE_DIR;
    }
    if (!@ftp_rmdir($ftp_stream, $upload_path . '/archibald_upload_check')) {
      return self::ERROR_PERMISSION_REMOVE_DIR;
    }

    //File upload validation
    file_put_contents("/tmp/archibald_upload_check", "test");
    if (!@ftp_put($ftp_stream, 'archibald_upload_check', '/tmp/archibald_upload_check', FTP_ASCII)) {
      return self::ERROR_PERMISSION_UPLOAD_FILE;
    }

    if (!@ftp_delete($ftp_stream, 'archibald_upload_check')) {
      return self::ERROR_PERMISSION_DELETE_FILE;
    }
    unlink('/tmp/archibald_upload_check');
    return TRUE;
  }

  /**
   * submit configuration form
   *
   * @param array $form
   * @param array $form_state
   */
  public function submitSettingsForm($form, &$form_state, $content_partner_id) {

    variable_set(
      'archibald_uploader_plugin_config_ftp_' . $content_partner_id,
      array_merge((array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id, array()), $form_state)
    );
  }

  /**
   * upload a single file to an ftp server
   *
   * @param int $content_partner_id the content partner id which want to upload
   * @param string $file_real_path
   *   full path to the file to upload
   * @param array $callback
   * @param object $objectToPublish
   *
   * @return object
   *
   *   code integer
   *     99   error ssl key problems
   *     501  error server throws wrong parameter
   *     405  error server throws file type not allowed
   *     413  error server throws file to big
   *     502  error server access denied
   *     xxx  error unknown error
   *     200  upload successfull
   *
   *   message string
   *     will displayed if code is not one of the defined
   *
   *   url string
   *     public url to access uploaded files
   *     and have to be set if code=200
   */
  public function uploadFile($content_partner_id, $file_real_path, $fid, $callback, &$object_to_publish) {

    $this->objectToPublish = &$object_to_publish;

    $this->parentCallbackMethode = $callback;

    $ftp_config = (array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id,
      array(
        'server' => '',
        'port' => '',
        'timeout' => '',
        'username' => '',
        'password' => '',
        'upload_path' => '',
        'passive' => '',
        'ssl' => '',
      )
    );

    $validate_result = $this->validateLogin(
      $ftp_config['server'],
      $ftp_config['port'],
      $ftp_config['timeout'],
      $ftp_config['username'],
      $ftp_config['password'],
      $ftp_config['upload_path'],
      $ftp_config['passive'],
      $ftp_config['ssl']
    );
    switch ($validate_result) {
      case TRUE:
        //all fine
        break;

      default:
        return (object)array('code' => 502);
    }

    $ch = curl_init();


    $port = '';
    if (!empty($ftp_config['port'])) {
      $port = ':' . $ftp_config['port'];
    }

    preg_match("/.*\/([^\/]+)$/is", $file_real_path, $matches);

    $schema = 'ftp';
    if ($ftp_config['ssl'] == TRUE) {
      $schema = 'ftps';
    }
    $url = $schema . '://' . $ftp_config['username'] . ':' . $ftp_config['password'] . '@' . $ftp_config['server'] . $port . '/' . $this->getUploadPath($ftp_config, $fid, $file_real_path);
    $this->mkdirDirs($ftp_config, $this->getUploadPath($ftp_config, $fid, $file_real_path, FALSE));
    curl_setopt($ch, CURLOPT_URL, $url);

    if ($ssl == TRUE) {
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
    }

    curl_setopt($ch, CURLOPT_UPLOAD, 1);

    $fp = fopen($file_real_path, 'r');
    curl_setopt($ch, CURLOPT_INFILE, $fp);
    curl_setopt($ch, CURLOPT_INFILESIZE, filesize($file_real_path));

    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Archibald/2.0 (@' . @$_SERVER['HTTP_HOST'] . ')');
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));

    if (defined('CURLOPT_PROGRESSFUNCTION')) {
      curl_setopt($ch, CURLOPT_NOPROGRESS, FALSE);
      curl_setopt($ch, CURLOPT_PROGRESSFUNCTION, array($this, 'uploadFileProgress'));
    }
    curl_exec($ch);
    $error_no = curl_errno($ch);
    curl_close($ch);

    if ($error_no != 0) {
      return (object)array('code' => 9999, 'message' => 'could not upload file');
    }

    $ftp_config['public_path'] = preg_replace("/\/+$/", "", $ftp_config['public_path']);

    $return = (object)array(
      'code' => 200,
      'message' => 'upload success',
      'url' => $ftp_config['public_path'] . '/' . $matches[1],
    );
    return $return;
  }

  /**
   * Deletes the given file
   *
   * @param object $file
   *   The file object
   *
   * @return boolean
   */
  public function deleteFile($file) {

    $ftp_config = (array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id,
      array(
        'server' => '',
        'port' => '',
        'timeout' => '',
        'username' => '',
        'password' => '',
        'upload_path' => '',
        'passive' => '',
        'ssl' => '',
      )
    );

    $res = ftp_connect($ftp_config['server'], $ftp_config['port']);
    if (ftp_login($res, $ftp_config['username'], $ftp_config['password'])) {
      if (ftp_delete($res, $this->getUploadPath($ftp_config, $file->fid, $file->uri))) {
        @ftp_rmdir($res, $this->getUploadPath($ftp_config, $file->fid, $file->uri, FALSE));
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Returns the upload path with or without the filename
   *
   * @param array $ftp_config
   *  The FTP-Configuration array
   *
   * @param int $fid
   *  the file id
   *
   * @param string $uri
   *  the filestring which includes the filename
   *
   * @param boolean $include_filename
   *  If the result includes the filename or just the directory
   *
   * @return string
   *  the upload path
   */
  private function getUploadPath($ftp_config, $fid, $uri, $include_filename = TRUE) {
    $path = "";

    if (!empty($ftp_config['upload_path']) && !preg_match("/^\/+$/", $ftp_config['upload_path'])) {
      $path = $ftp_config['upload_path'];
      $path = preg_replace("/^\/+/", "", $path);
      $path = preg_replace("/\/+$/", "", $path);
    }
    preg_match("/.*\/([^\/]+)$/is", $uri, $matches);

    if ($include_filename) {
      return $path . '/' . $fid . '/' . $matches[1];
    }

    return preg_replace("/\/+/", "/", $path . '/' . $fid);
  }


  /**
   * Generates the specified path on the ftp server
   *
   * @param array $ftp_config
   *  The FTP-Configuration array
   *
   * @param string $path
   *  The path to be created
   */
  private function mkdirDirs($ftp_config, $path) {
    $res = ftp_connect($ftp_config['server'], $ftp_config['port']);
    ftp_login($res, $ftp_config['username'], $ftp_config['password']);

    $dir = "/";
    foreach (explode("/", $path) as $entry) {
      if (empty($entry)) {
        continue;
      }
      $dir .= $entry;
      @ftp_mkdir($res, $dir);
      $dir .= '/';
    }
  }
}

==> archibald-7.x-2.1.tar.gz: archibald/includes/file_uploader/uploader.ftp.class.php <==
<?php

/**
 * @file
 * Provides the FTP upload handler
 */

/**
 * file uploader to educa file server
 * used by lom publish
 *
 * @author Heiko Henning h.henning@educa.ch
 */
class ArchibaldUploaderFtp extends ArchibaldUploaderCurlBased implements ArchibaldUploaderInterface {

  /**
   * Error constances
   */
  const ERROR_INVALID_PORT = 0;
  const ERROR_INVALID_CONNECTION_TIMEOUT = 1;
  const ERROR_CONNECTION_TIMEOUT = 2;
  const ERROR_INVALID_CREDENTIALS = 3;
  const ERROR_INVALID_UPLOAD_PATH = 4;
  const ERROR_PERMISSION_CREATE_DIR = 5;
  const ERROR_PERMISSION_REMOVE_DIR = 6;
  const ERROR_PERMISSION_UPLOAD_FILE = 7;
  const ERROR_PERMISSION_DELETE_FILE = 8;

  /**
   * get the name of the uploader
   *
   * @return array
   *  first value is the computer readable code
   *  second value is the human readable text
   */
  public function getUploaderName() {
    return array('ftp' => t('FTP server'));
  }

  /**
   * get the description of the uploader
   *
   * @return string
   *   html
   */
  public function getUploaderDescription() {
    return t('Uploads files to the configured FTP server.');
  }

  /**
   * get settings configuration form
   *
   * @param array $form
   * @param array $form_state
   *
   * @return array
   *   drupal form
   */
  public function getSettingsForm($form, &$form_state, $content_partner_id) {

    $saved_values = array_merge(
      array(
        'server' => 'localhost',
        'port' => '21',
        'ssl' => '0',
        'timeout' => '60',
        'username' => '',
        'password' => '',
        'passive' => '0',
        'upload_path' => '/',
        'public_path' => '/',
      ),
      (array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id, array())
    );

    $settings_form['server'] = array(
      '#type' => 'textfield',
      '#title' => t('Servername/IP-Address'),
      '#required' => TRUE,
      '#default_value' => $saved_values['server'],
    );


    $settings_form['port'] = array(
      '#type' => 'textfield',
      '#title' => t('Port'),
      '#default_value' => $saved_values['port'],
    );

    $settings_form['ssl'] = array(
      '#type' => 'checkbox',
      '#title' => t('SSL'),
      '#return_value' => 1,
      '#default_value' => $saved_values['ssl'],
    );

    $settings_form['passive'] = array(
      '#type' => 'checkbox',
      '#title' => t('Passive-mode'),
      '#return_value' => 1,
      '#default_value' => $saved_values['passive'],
    );

    $settings_form['timeout'] = array(
      '#type' => 'textfield',
      '#title' => t('Connection timeout'),
      '#default_value' => $saved_values['timeout'],
    );


    $settings_form['username'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#default_value' => $saved_values['username'],
    );

    $settings_form['password'] = array(
      '#type' => 'textfield',
      '#title' => t('Password'),
      '#default_value' => $saved_values['password'],
    );

    $settings_form['upload_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Upload path on FTP server'),
      '#required' => TRUE,
      '#default_value' => $saved_values['upload_path'],
    );


    $settings_form['public_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Public URL and path on FTP server (example: http://my-url.ftp.info/files/'),
      '#description' => 'Please enter the web based public URL and path where the files will be available after uploading',
      '#required' => TRUE,
      '#default_value' => $saved_values['public_path'],
    );

    return $settings_form;
  }

  /**
   * validate configuration form
   *
   * @param array $form
   * @param array $form_state
   */
  public function validateSettingsForm($form, &$form_state) {
    if (!isset($form_state['port'])) {
      return;
    }

    if (empty($form_state['username']) && empty($form_state['password'])) {
      $form_state['username'] = "anonymous";
      $form_state['password'] = "anonymous@anonymous.de";
    }

    $validate_result = $this->validateLogin(
      $form_state['server'], $form_state['port'], $form_state['timeout'], $form_state['username'], $form_state['password'], $form_state['upload_path'], $form_state['passive'], $form_state['ssl']
    );

    if ($validate_result !== TRUE) {
      switch ($validate_result) {
        case self::ERROR_INVALID_PORT:
          return form_set_error('file_uploader_config][port', t('Invalid port, please use just numbers'));

        case self::ERROR_INVALID_CONNECTION_TIMEOUT:
          return form_set_error('file_uploader_config][timeout', t('Invalid connection timeout, please use just numbers'));

        case self::ERROR_CONNECTION_TIMEOUT:
          return form_set_error('file_uploader_config', t('Cannot connect to server, please check your server settings.'));

        case self::ERROR_INVALID_CREDENTIALS:

          return form_set_error('file_uploader_config', t('Invalid credentials, please check your username and password'));

        case self::ERROR_INVALID_UPLOAD_PATH:
          return form_set_error('file_uploader_config][upload_path', t('Could not find upload path, please check your settings'));

        case self::ERROR_PERMISSION_CREATE_DIR:
          return form_set_error('file_uploader_config][upload_path', t('The current user has no permission to create directories at the specific upload path, please check your settings.'));

        case self::ERROR_PERMISSION_REMOVE_DIR:
          return form_set_error('file_uploader_config][upload_path', t('The current user has no permission to create directories at the specific upload path, please check your settings.'));

        case self::ERROR_PERMISSION_UPLOAD_FILE:
          return form_set_error('file_uploader_config][upload_path', t('The current user has no permission to upload files at the specific upload path, please check your settings.'));

        case self::ERROR_PERMISSION_DELETE_FILE:
          return form_set_error('file_uploader_config][upload_path', t('The current user has no permission to delete files at the specific upload path, please check your settings.'));

        default:
          return form_set_error('file_uploader_config', t('An unknown error occured'));
      }
    }

    if (!preg_match("/^https?:\/\/(www.)?/", $form_state['public_path'])) {
      return form_set_error('file_uploader_config][public_path', t('The public path must be a full URL including http(s)://'));
    }
  }

  /**
   *
   * @param string $server the servername or the server address
   * @param int $port the port number
   * @param int $timeout the timeout for connecting
   * @param string $username the username
   * @param string $password the password
   * @param string $upload_path the path where to upload
   * @param boolean $passive TRUE to use passive mode
   * @param boolean $ssl TRUE to enable ssl
   *
   * @return mixed TRUE on success or the error number as an integer (one of uploader_ftp::ERROR_*)
   */
  private function validateLogin($server, $port, $timeout, $username, $password, $upload_path, $passive, $ssl) {
    if ((INT) $port . '' != $port . '') {
      return self::ERROR_INVALID_PORT;
    }

    if ((INT) $timeout . '' != $timeout . '') {
      return self::ERROR_INVALID_CONNECTION_TIMEOUT;
    }

    //server connection validation
    if (!empty($ssl)) {
      $ftp_stream = @ftp_ssl_connect($server, $port, $timeout);
    }
    else {
      $ftp_stream = @ftp_connect($server, $port, $timeout);
    }

    if (!$ftp_stream) {
      return self::ERROR_CONNECTION_TIMEOUT;
    }

    //set passive mode validation
    if (!empty($passive)) {
      ftp_pasv($ftp_stream, TRUE);
    }

    //credentials validation
    if (!@ftp_login($ftp_stream, $username, $password)) {
      return self::ERROR_INVALID_CREDENTIALS;
    }

    //upload path exist validation
    if (!@ftp_chdir($ftp_stream, $upload_path)) {
      return self::ERROR_INVALID_UPLOAD_PATH;
    }

    //Create directory validation
    $perm_check = @ftp_mkdir($ftp_stream, $upload_path . '/archibald_upload_check');
    if (empty($perm_check)) {
      return self::ERROR_PERMISSION_CREATE_DIR;
    }
    if (!@ftp_rmdir($ftp_stream, $upload_path . '/archibald_upload_check')) {
      return self::ERROR_PERMISSION_REMOVE_DIR;
    }

    //File upload validation
    file_put_contents('/tmp/archibald_upload_check', 'test');
    if (!@ftp_put($ftp_stream, 'archibald_upload_check', '/tmp/archibald_upload_check', FTP_ASCII)) {
      return self::ERROR_PERMISSION_UPLOAD_FILE;
    }

    if (!@ftp_delete($ftp_stream, 'archibald_upload_check')) {
      return self::ERROR_PERMISSION_DELETE_FILE;
    }
    unlink('/tmp/archibald_upload_check');
    return TRUE;
  }

  /**
   * submit configuration form
   *
   * @param array $form
   * @param array $form_state
   */
  public function submitSettingsForm($form, &$form_state, $content_partner_id) {

    variable_set(
      'archibald_uploader_plugin_config_ftp_' . $content_partner_id,
      array_merge((array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id, array()), $form_state)
    );
  }

  /**
   * upload a single file to an ftp server
   *
   * @param int $content_partner_id the content partner id which want to upload
   * @param string $file_real_path
   *   full path to the file to upload
   * @param array $callback
   * @param object $objectToPublish
   *
   * @return object
   *
   *   code integer
   *     99   error ssl key problems
   *     501  error server throws wrong parameter
   *     405  error server throws file type not allowed
   *     413  error server throws file to big
   *     502  error server access denied
   *     xxx  error unknown error
   *     200  upload successfull
   *
   *   message string
   *     will displayed if code is not one of the defined
   *
   *   url string
   *     public url to access uploaded files
   *     and have to be set if code=200
   */
  public function uploadFile($content_partner_id, $file_real_path, $fid, $callback, &$object_to_publish) {

    $this->objectToPublish = &$object_to_publish;

    $this->parentCallbackMethode = $callback;

    $ftp_config = (array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id,
      array(
        'server' => '',
        'port' => '',
        'timeout' => '',
        'username' => '',
        'password' => '',
        'upload_path' => '',
        'passive' => '',
        'ssl' => '',
      )
    );

    $validate_result = $this->validateLogin(
      $ftp_config['server'],
      $ftp_config['port'],
      $ftp_config['timeout'],
      $ftp_config['username'],
      $ftp_config['password'],
      $ftp_config['upload_path'],
      $ftp_config['passive'],
      $ftp_config['ssl']
    );
    switch ($validate_result) {
      case TRUE:
        //all fine
        break;

      default:
        return (object)array('code' => 502);
    }

    $ch = curl_init();


    $port = '';
    if (!empty($ftp_config['port'])) {
      $port = ':' . $ftp_config['port'];
    }

    preg_match("/.*\/([^\/]+)$/is", $file_real_path, $matches);

    $schema = 'ftp';
    if ($ftp_config['ssl'] == TRUE) {
      $schema = 'ftps';
    }
    $url = $schema . '://' . $ftp_config['username'] . ':' . $ftp_config['password'] . '@' . $ftp_config['server'] . $port . '/' . $this->getUploadPath($ftp_config, $fid, $file_real_path);
    $this->mkdirDirs($ftp_config, $this->getUploadPath($ftp_config, $fid, $file_real_path, FALSE));
    curl_setopt($ch, CURLOPT_URL, $url);

    if ($ssl == TRUE) {
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
    }

    curl_setopt($ch, CURLOPT_UPLOAD, 1);

    $fp = fopen($file_real_path, 'r');
    curl_setopt($ch, CURLOPT_INFILE, $fp);
    curl_setopt($ch, CURLOPT_INFILESIZE, filesize($file_real_path));

    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Archibald/2.0 (@' . @$_SERVER['HTTP_HOST'] . ')');
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));

    if (defined('CURLOPT_PROGRESSFUNCTION')) {
      curl_setopt($ch, CURLOPT_NOPROGRESS, FALSE);
      curl_setopt($ch, CURLOPT_PROGRESSFUNCTION, array($this, 'uploadFileProgress'));
    }
    curl_exec($ch);
    $error_no = curl_errno($ch);
    curl_close($ch);

    if ($error_no != 0) {
      return (object)array('code' => 9999, 'message' => 'could not upload file');
    }

    $ftp_config['public_path'] = preg_replace("/\/+$/", "", $ftp_config['public_path']);

    $return = (object)array(
      'code' => 200,
      'message' => 'upload success',
      'url' => $ftp_config['public_path'] . '/' . $matches[1],
    );
    return $return;
  }

  /**
   * Deletes the given file
   *
   * @param object $file
   *   The file object
   *
   * @return boolean
   */
  public function deleteFile($file) {

    $ftp_config = (array) variable_get('archibald_uploader_plugin_config_ftp_' . $content_partner_id,
      array(
        'server' => '',
        'port' => '',
        'timeout' => '',
        'username' => '',
        'password' => '',
        'upload_path' => '',
        'passive' => '',
        'ssl' => '',
      )
    );

    $res = ftp_connect($ftp_config['server'], $ftp_config['port']);
    if (ftp_login($res, $ftp_config['username'], $ftp_config['password'])) {
      if (ftp_delete($res, $this->getUploadPath($ftp_config, $file->fid, $file->uri))) {
        @ftp_rmdir($res, $this->getUploadPath($ftp_config, $file->fid, $file->uri, FALSE));
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Returns the upload path with or without the filename
   *
   * @param array $ftp_config
   *  The FTP-Configuration array
   *
   * @param int $fid
   *  the file id
   *
   * @param string $uri
   *  the filestring which includes the filename
   *
   * @param boolean $include_filename
   *  If the result includes the filename or just the directory
   *
   * @return string
   *  the upload path
   */
  private function getUploadPath($ftp_config, $fid, $uri, $include_filename = TRUE) {
    $path = "";

    if (!empty($ftp_config['upload_path']) && !preg_match("/^\/+$/", $ftp_config['upload_path'])) {
      $path = $ftp_config['upload_path'];
      $path = preg_replace("/^\/+/", "", $path);
      $path = preg_replace("/\/+$/", "", $path);
    }
    preg_match("/.*\/([^\/]+)$/is", $uri, $matches);

    if ($include_filename) {
      return $path . '/' . $fid . '/' . $matches[1];
    }

    return preg_replace("/\/+/", "/", $path . '/' . $fid);
  }


  /**
   * Generates the specified path on the ftp server
   *
   * @param array $ftp_config
   *  The FTP-Configuration array
   *
   * @param string $path
   *  The path to be created
   */
  private function mkdirDirs($ftp_config, $path) {
    $res = ftp_connect($ftp_config['server'], $ftp_config['port']);
    ftp_login($res, $ftp_config['username'], $ftp_config['password']);

    $dir = "/";
    foreach (explode("/", $path) as $entry) {
      if (empty($entry)) {
        continue;
      }
      $dir .= $entry;
      @ftp_mkdir($res, $dir);
      $dir .= '/';
    }
  }
}
